/*
 * Exemplo de implementacao do algoritmo RSA em Java
 * Para as chaves geradas, solicita e cifra mensagens M, ate ler M em branco
 * Por Marcio H. C. d'Avila <mhavila@inet.com.br>, 2001
 */
import java.math.BigInteger;
import java.security.SecureRandom;
import java.io.*;

public class Rsa
{
  private BigInteger n, d, e;

  public Rsa(int bitlen)
  {
    // Escolher p e q numeros primos grandes (numero de bits = bitlen/2)
    // BigInteger(b,c,r) = inteiro positivo randomico de b bits provavelmente primo, com
    // chance de erro de 1 em 2^c, tomando r como fonte de bits para testes de primalidade
    SecureRandom r = new SecureRandom();
    BigInteger p = new BigInteger(bitlen / 2, 100, r);
    System.out.println("Gerei p");
    BigInteger q = new BigInteger(bitlen / 2, 100, r);
    System.out.println("Gerei q");

    // Calcular n = p * q
    n = p.multiply(q);
    System.out.println("Calculei n");

    // Calcular o quociente euleriano fi(n) = (p - 1)(q - 1)
    BigInteger fi_n = p.subtract(BigInteger.ONE).multiply(q.subtract(BigInteger.ONE));
    System.out.println("Calculei fi(n)");

    // Encontrar e que seja primo relativo a fi(n), ou MDC(e, fi(n)) = 1
    // A estrategia: encontrar um e pequeno, tentando inicialmente o 3
    // e depois seguindo pelos demais inteiros mpares positivos
    e = new BigInteger("3");
    while(fi_n.gcd(e).intValue() > 1) e = e.add(new BigInteger("2"));
    System.out.println("Encontrei e");

    // Calcular d como o inverso multiplicativo de e mod fi(n), ou d*e = 1 mod fi(n)
    d = e.modInverse(fi_n);
    System.out.println("Calculei d");
  }

  public BigInteger encrypt(BigInteger message)
  {
    return message.modPow(e, n);
  }

  public BigInteger decrypt(BigInteger message)
  {
    return message.modPow(d, n);
  }

  static public void main(String [] args)
  {
    System.out.println("Inicializando RSA...");
    Rsa rsa = new Rsa(1024);
    System.out.println("");
    System.out.println("n = " + rsa.n);
    System.out.println("e = " + rsa.e);
    System.out.println("d = " + rsa.d);
    System.out.println("");

    System.out.println("Encriptando e Decriptando...");
    BigInteger M;
    String s;
    InputStreamReader isr = new InputStreamReader(System.in);
    BufferedReader br = new BufferedReader(isr);
    System.out.print("M = ");
    try {
      s = br.readLine();
    } catch(IOException e) {
      s = "";
    }
    while(! s.equals("")) {
      M = new BigInteger(s);
      BigInteger C = rsa.encrypt(M);
      System.out.println("C = " + C);
      BigInteger m = rsa.decrypt(C);
      System.out.println("m = " + m);

      System.out.println("");
      System.out.print("M = ");
      try {
        s = br.readLine();
      } catch(IOException e) {
        s = "";
      }
    } // while
  }
} // class Rsa

